import PropTypes from 'prop-types';
import React from 'react';
import Notebook from './component';
import Sandbox from './sandbox';

export default class StandaloneApp extends React.Component {
  static propTypes = {
    content: PropTypes.string,
    blockNumber: PropTypes.number,
    loaderPath: PropTypes.string.isRequired,
    sandbox: PropTypes.object,
    onChange: PropTypes.func
  };

  static defaultProps = {
    content: '',
    blockNumber: 0,
    loaderPath: 'https://unpkg.com/api-notebook-react/static/vs',
    sandbox: new Sandbox(),
    onChange: () => {}
  };

  constructor(props) {
    super(props);

    this.state = {
      notebookState: {
        content: this.props.content
      }
    };
  }

  handleChange = (notebookState) => {
    this.setState({
      notebookState
    }, () => {
      this.props.onChange(notebookState);
    });
  }

  handleExecute = () => {
    const {
      notebookState
    } = this.state;
    const {
      blockNumber,
      sandbox
    } = this.props;

    this.handleChange({ ...notebookState, isLoading: true });

    return sandbox.execute(notebookState.content, blockNumber)
      .then((exec) => {
        this.handleChange({ ...notebookState, exec, isLoading: false });
      });
  }

  render() {
    const {
      blockNumber,
      loaderPath
    } = this.props;
    const {
      notebookState
    } = this.state;

    return (
      <Notebook
        blockNumber={blockNumber}
        loaderPath={loaderPath}
        notebookState={notebookState}
        onChange={this.handleChange}
        onExecute={this.handleExecute}
      />
    );
  }
}
